/**-----------------------------------------------------------------------------------------
* Copyright © 2025 Progress Software Corporation. All rights reserved.
* Licensed under commercial license. See LICENSE.md in the project root for more information
*-------------------------------------------------------------------------------------------*/
/**
 * The type of the Grid pager.
 *
 * The available values are:
 * * `numeric`&mdash;Buttons with numbers.
 * * `input`&mdash;Input for typing the page number.
 *
 * @example
 * ```html
 * <kendo-grid [pageable]="{ type: 'numeric' }">
 *   <kendo-grid-column field="ProductID"></kendo-grid-column>
 *   ...
 * </kendo-grid>
 * ```
 */
export type PagerType = 'numeric' | 'input';
/**
 * The position of the Grid pager.
 *
 * The available values are:
 * * `top`&mdash;The pager is rendered above the Grid headers.
 * * `bottom`&mdash;(Default) The pager is rendered below the data table.
 * * `both`&mdash;Two pagers are rendered - one above the Grid headers, and one - below the data table.
 *
 * @example
 * ```html
 * <kendo-grid [pageable]="{ position: 'both' }">
 *   <kendo-grid-column field="ProductID"></kendo-grid-column>
 *   ...
 * </kendo-grid>
 * ```
 */
export type PagerPosition = 'top' | 'bottom' | 'both';
/**
 * The pager settings of the Grid ([see example](slug:paging_grid_settings)).
 */
export interface PagerSettings {
    /**
     * Sets the maximum numeric buttons count before the buttons are collapsed.
     */
    buttonCount?: number;
    /**
     * Toggles the information about the current page and the total number of records.
     */
    info?: boolean;
    /**
     * Defines the type of the Grid pager.
     */
    type?: PagerType;
    /**
     * Shows a menu for selecting the page size.
     */
    pageSizes?: boolean | Array<number>;
    /**
     * Toggles the **Previous** and **Next** buttons.
     */
    previousNext?: boolean;
    /**
     * Toggles the built-in responsive behavior of the Pager.
     * Available in version `5.0.0` and above ([see example](slug:paging_grid_settings)).
     */
    responsive?: boolean;
    /**
     * Defines the Pager position relative to the Grid data table.
     * The possible values are 'top', 'bottom', and 'both' ([see example](slug:paging_grid_settings)).
     */
    position?: PagerPosition;
}
/**
 * The interface of the `PageSizeItem` options of the `PagerPageSizesComponent`.
 */
export interface PageSizeItem {
    /**
     * The text that will be displayed for each option in the pager's PageSize selector.
     */
    text: string;
    /**
     * The value used as the page size. When set to `all`, the page size will be set to match the Grid data `total`.
     */
    value: number | 'all';
}
/**
 * @hidden
 */
export declare const normalize: (settings: any) => any;
